function results = ScanN(y,C,nv,e,k,Nrep,info)
%% ---------------------------------------------------------------------
%  Inputs
%    Nrep    : The number of Monte Carlo replications, e.g., 999, 9999
%    C       : Coordenate vector Rx2.
%    y       : Data vector Rx1
%    nv      : maximun number of neighbors inside cluster.
%    e       : Cluster type (e=0) circular (e=1) elliptic.
%    k       : Looking for only high incidence cluster (k=0); only low (k=1); both (k=2).
%    info    : Optional argument
%
%  Outputs
%    pvalue  : Monte Carlo permutational bootstraping p-value
%    Scan    : Value Scan statisitic
%    size    : Number of element inside of Most Likely Cluster (MLC)
%    Coor    : Coordenates of the center of MLC
%    angle   : Angle of ellipse of MLC
%    shape   : Shape of ellipse of MLC
%    Elem    : Elements (Id's) inside MLC
%    MLboots : Value of Lik in each boots
%    Clus    : Information about secondary cluster
% ----------------------------------------------------------------------------
%   
% Fernando A. Lopez Hernandez, 20/02/2012
% Dpto. Metodos Cuantitativos e Informaticos
% Universidad Politecnica de Cartagena
% e-mail: fernando.lopez@upct.es
%
if nargin<7
    info.print='no';
    info.plot='no';
end

R=length(y);

%% Selection of type of windows and get all ellipses
if e==0, % Only Circular Windows
NNN=[(1:R)' find_neighbors(C(:,1),C(:,2),nv-1) (1:R)' zeros(R,2)];
elseif e==1, % Elliptic Windows
E=nn_ellipse(C(:,1),C(:,2),nv);
NNN=[E.nn.eq (1:R)' zeros(R,2)];
for p=10:10:180 % Differents Angles
for s=1:5 % Differents Shape
% Las tres ultimas cifras caracterizan la elipse: Centro Angulo Shape
NNN=vertcat(NNN,[E.nne(p/10,s).eq (1:R)' p*ones(R,1) s*ones(R,1)]); % pool all ellipses
end
end
end
% Selec only the first time is in the list 
[~, b]=unique(sort(NNN(:,1:nv),2),'rows','first');
NNN=NNN(b,:);

%%
yNNN=y(NNN(:,1:nv));
Ra=size(NNN,1);
LL0=[0 0 0];
stdY=std(y,1);
mY=mean(y);
X=sum(y);
LogLik=[];
nz=[];

%% Permutational bootstrapping
MLboots=zeros(Nrep,1);
paso=R*log(stdY)+sum((y-mY).^2)/(2*stdY^2)-(R/2);

parfor f=1:Nrep
MCML=-10000000;
Y=y(randperm(R));
YNNN=Y(NNN(:,1:nv));
cs=cumsum(YNNN.^2,2);
cs2=cumsum(YNNN,2);
cm2=cs2./cumsum(ones(Ra,nv),2);
cm=(X-cs2)./(R-cumsum(ones(Ra,nv),2));
I=cumsum(ones(Ra,nv),2);
LL=paso-R*log(sqrt((cs-2*cs2.*cm2+I.*cm2.^2+(Y'*Y-cs)-2*(X-cs2).*cm+(R-I).*cm.^2)/R));

% Type of Cluster: High/Low/Both
if k==0, %cluster of high incidence
LL=LL.*(cm2>cm);
elseif k==1, % cluster of low incidence
LL=LL.*(cm2<cm);
end

MLboots(f)=max(max(LL));%MCML;

end

limit=prctile(MLboots,80);

%% Evaluate the correct value
for i=1:nv-1
nz=i+1;
yNN=yNNN(:,1:nz);
xz=sum(yNN,2);
landaz=(X-xz)/(R-nz);
muz=xz/nz;
sz=(sum(yNN.^2,2)-2*xz.*muz+nz*muz.^2+(sum(y.^2)-sum(yNN.^2,2)) -2*(X-xz).*landaz+(R-nz)*landaz.^2)/R;
L=R*log(stdY)+sum((y-mY).^2)/(2*stdY^2)-(R/2)-R*log(sqrt(sz));

%% Select type of cluster: High/Low/Both
if k==0, % Look for only high incidence cluster
L=L.*(muz>landaz);
elseif k==1, % Look for only low incidence cluster
L=L.*(muz<landaz);    
elseif k==2, % Look for cluster of high or low incidence
    L=L;
end
LogLik=horzcat(LogLik,L);
% The maximun values of LogLik
LL=[L (1:Ra)' nz*ones(Ra,1)];
LL=sortrows([LL0;LL],[1 2]);

alfa=sum(LL(:,1)>limit)+10;
%LL0=LL(end-min(R-1,length(LL)-1):end,:);% LL0=[valor de Lik ; fila de NNN en la que esta el cluster; n de elemetos de la fila que debo tomar]
LL0=LL(end-alfa:end,:);
end

results.Lik=LL0(end,1);% Log Lik Value
results.Elem=NNN(LL0(end,2),1:LL0(end,3));% IDs of Most Likely Cluster (MLC)
results.size=length(results.Elem);% size of cluster
results.Coor=C(results.Elem(1,1),:);% coordenates of the center of cluster
results.shape=NNN(LL0(end,2),end);% Ellipse excentricity
results.angle=NNN(LL0(end,2),end-1);% Ellipse angle
results.m_in=mean(y(results.Elem));% Mean inside MLC
results.m_out=(R*mY-results.size*results.m_in)/(R-results.size);% Mean outside MLC
results.m=mY;
MLf=results.Lik;

%% Non-overlapping secondary and significative cluster
listanegra=NNN(LL0(end,2),1:LL0(end,3));
nsc=0;
Clus=[];

for i=1:length(LL0)-1
    if sum(ismember(NNN(LL0(end-i,2),1:LL0(end-i,3)),listanegra))==0;
        listanegra=[listanegra,NNN(LL0(end-i,2),1:LL0(end-i,3))];
        nsc=nsc+1;
        Clus(nsc).Elem=NNN(LL0(end-i,2),1:LL0(end-i,3));
        Clus(nsc).Lik=LL0(end-i,1);
        Clus(nsc).pvalue=sum(MLboots>LL0(end-i,1))/(Nrep+1);
        Clus(nsc).size=length(Clus(nsc).Elem);
        Clus(nsc).m_in=mean(y(Clus(nsc).Elem));
        Clus(nsc).m_out=(R*mY-Clus(nsc).size*Clus(nsc).m_in)/(R-Clus(nsc).size);
    end
end
% Output secondary cluster
results.Clus=Clus;

%% Output
results.pvalue=sum(MLboots>MLf)/(Nrep+1);
results.MLboots=MLboots;
results.Nrep=Nrep;

if strcmp(info.plot,'yes')
    close all
    
     figure('Color',[1 1 1]);
    hold on
    axis off
   
    plot(C(:,1),C(:,2),'ob','MarkerSize',12)
    plot(C(results.Elem,1),C(results.Elem,2),'*r','MarkerSize',12,'MarkerEdgeColor','r','MarkerEdgeColor','r')
    plot(C(results.Elem(1,1),1),C(results.Elem(1,1),2),'sb','MarkerSize',20)
    if ~isempty(Clus)
        for i=1:length(Clus)
    plot(C(Clus(i).Elem,1),C(Clus(i).Elem,2),'m+','MarkerSize',12)
        end
    end
    hold off
    
end

if strcmp(info.print,'yes')
   fprintf(1,'\n')
   fprintf(1,'  Sample size               (N) = %9.0f\n',R)
   fprintf(1,'  Maximun cluster size     (nv) = %9.0f\n',nv)
   fprintf(1,'  Type of windows           (e) = %9.0f\n',e)
   fprintf(1,'  Incidence Cluster         (k) = %9.0f\n',k)
   fprintf(1,'  Global mean                   = %9.2f\n',mean(y))
   fprintf(1,'\n')
   fprintf(1,'  MOST LIKELY CLUSTER\n')
   fprintf(1,'  Elements IDs                ')
   end
   disp(num2str(results.Elem))
   fprintf(1,'  Number of cases       = %9.0f\n',results.size)
   fprintf(1,'  Mean inside           = %9.2f\n',results.m_in)
   fprintf(1,'  Mean outside          = %9.2f\n',results.m_out)
   fprintf(1,'  Log Lik               = %9.2f\n',results.Lik)
   fprintf(1,'  p-value               = %9.3f\n',results.pvalue)
   if ~isempty(Clus)
   fprintf(1,'\n')
   fprintf(1,'  SECONDARY CLUSTERS\n')
   fprintf(1,'\n')
   for i=1:length(Clus)
       fprintf(1,'  Elements IDs                ')
   disp(num2str(Clus(i).Elem))
   fprintf(1,'  Number of cases       = %9.0f\n',Clus(i).size)
   fprintf(1,'  Mean inside           = %9.2f\n',Clus(i).m_in)
   fprintf(1,'  Mean outside          = %9.2f\n',Clus(i).m_out)
   fprintf(1,'  Log Lik               = %9.2f\n',Clus(i).Lik)
   fprintf(1,'  p-value               = %9.3f\n',Clus(i).pvalue)
   fprintf(1,'\n')
   end
   end
   
end

